<?php 

namespace Schaffen\SerasaRemessa\Http\Advanced\PF;
use GuzzleHttp\Client;
use Schaffen\SerasaRemessa\Http\Advanced\PF\Blocks\ClosingBlock;
use Schaffen\SerasaRemessa\Http\Advanced\PF\Blocks\MainBlock;
use Schaffen\SerasaRemessa\Http\Advanced\PF\Blocks\OpeningBlock;
use Schaffen\SerasaRemessa\Http\Advanced\PF\Blocks\RegN001;
use Schaffen\SerasaRemessa\Http\Advanced\PF\Blocks\RegN003;
use Schaffen\SerasaRemessa\Http\Advanced\PF\Blocks\RegP002;
use Schaffen\SerasaRemessa\Http\Advanced\PF\Blocks\RegP006;

class Query {
      
    /**
     * @var OpeningBlock
     */
    private $openingBlock;

    /**
     * @var MainBlock 
     */
    private $mainBlock;

    /**
     * @var RegP002
     */
    private $regP002;

    /**
     * @var RegP006
     */
    private $regP006;
    
    /**
     * @var ClosingBlock
     */
    private $closingBlock;

    /**
     * @param string $logon Logon do usuário na API do Serasa 
     * @param string $password Senha do usuário na API do Serasa 
     * @param array $config  Lista de configurações para gerar a requisição
     */
    public function __construct($logon, $password, $config) {
        $this->openingBlock = new OpeningBlock($logon, $password);
        $this->mainBlock = new MainBlock($config);

        $this->setP002();
        $this->regP006 = new RegP006([]);

        $this->closingBlock = new ClosingBlock();
    }

    /**
     * Configura o bloco P002 para carregar as opções extra requisitadas
     * ** TODO: Adicionar opções de configuração para selecionar informações adicionais. **
     * ** Será necesário gerenciar a quantidade de blocos P002 **
     */
    public function setP002() {
        /**
         * Valores fixos por hora
         * BPTM -> Comprometimento de Renda
         * BPPG -> Capacidade de Pagamento 
         * BPHM –> Score Positivo PF
         * BPRD -> Renda estimada
         */
        // $this->regP002 = new RegP002("BPTM", "BPPG", "BPHM", "BPRD");
        $this->regP002 = new RegP002("BPCA", "BPRD", "BPHMHSPN");
    }


    /**
     * @return string bloco de abertura com Logon e Senha
     */
    private function makeOpeningBlock() {
        return $this->openingBlock->toString();
    }

    /**
     * @return string bloco principal B49C
     */
    private function makeMainBlock() {
        return $this->mainBlock->toString();
    }
    
    /**
     * @return string Bloco P002
     */
    private function makeRegP002() {
        return $this->regP002->toString();
    }

    /**
     * @return string Bloco P006
     */
    private function makeRegP006() {
        return $this->regP006->toString();
    }

    /**
     * @return string Bloco de indicação de encerramento do parâmetro
     */
    private function makeClosingBlock() {
        return $this->closingBlock->toString();
    }

    /**
     * Gera o paramêtro 'p' da requisição a partir dos blocos cadastrados
     * @return string Parâmtro da requisição formatado 
     */
    public function make() {
        $openingBlock = $this->makeOpeningBlock();
        $mainBlock = $this->makeMainBlock();
        $regP002 = $this->makeRegP002();
        $regP006 = $this->makeRegP006();

        $closingBlock = $this->makeClosingBlock();
        
        return $openingBlock . $mainBlock . $regP002 . $regP006 . $closingBlock;
    }

    /** 
     * Método responsável por enviar a requsição à API Serasa Experian
     * @param string  $url URL da API
     * @return array | bool Resultado da requisição se sucesso ou false em caso de erro 
     * ** TODO: aprimorar retorno de erros ** 
    */
    public function send($url) {
        try {
            $p = $this->make();
            $client = new Client();
            $body = $client->request('POST', $url, ['query' => ['p' => $p]])->getBody();
            // dd($body);
            return [
                'translate' => Response::load($body)->translate(),
                'body' => $body,
                'params' => 'verbo=POST, url=' . $url . ', p=' . $p
            ];
        } catch(\Exception $e) {
            return false;
        }
    }

    public function translateBody($body) {
        return Response::load($body)->translate();
    }
}